@@ -8,7 +8,7 @@ package UR::Context;
 use strict;
 use warnings;
 
-our $VERSION = "0.46"; # UR $VERSION;
+our $VERSION = "0.35"; # UR $VERSION;
 
 
 # A wrapper around the method of the same name in UR::DataSource::* to iterate over the
@@ -200,7 +200,6 @@ sub _create_secondary_loading_closures {
             my %foreign_property_name_map;
             my @this_property_joins = $property->_resolve_join_chain();
             foreach my $join ( @this_property_joins ) {
-                last if ($join->{foreign_class}->isa('UR::Value') and $join eq $this_property_joins[-1]);
                 my @source_names = @{$join->{'source_property_names'}};
                 my @foreign_names = @{$join->{'foreign_property_names'}};
                 @foreign_property_name_map{@foreign_names} = @source_names;
@@ -217,17 +216,10 @@ sub _create_secondary_loading_closures {
                         my $column_position = $tmpl->{'column_positions'}->[$i];
                         # What are the types involved?
                         my $primary_query_column_name = $foreign_property_name_map{$property_name};
-                        my $primary_property_class_meta = $primary_template->{'class_name'}->__meta__;
-                        my $primary_property_meta = $primary_property_class_meta->property_meta_for_name($primary_query_column_name);
-                        unless ($primary_property_meta) {
-                            Carp::croak("Can't resolve property metadata for property '$primary_query_column_name' of class ".$primary_template->{'class_name'});
-                        }
-
-                        my $secondary_class_meta = $secondary_template->{'class_name'}->__meta__;
-                        my $secondary_property_meta = $secondary_class_meta->property_meta_for_name($property_name);
-                        unless ($secondary_property_meta) {
-                            Carp::croak("Can't resolve property metadata for property '$property_name' of class ".$secondary_template->{'class_name'});
-                        }
+                        my $primary_property_meta = UR::Object::Property->get(class_name => $primary_template->{'class_name'},
+                                                                              property_name => $primary_query_column_name);
+                        my $secondary_property_meta = UR::Object::Property->get(class_name => $secondary_template->{'class_name'},
+                                                                                property_name => $property_name);
 
                         my $comparison_type;
                         if ($primary_property_meta->is_numeric && $secondary_property_meta->is_numeric) {
@@ -392,6 +384,7 @@ sub _create_import_iterator_for_underlying_context {
     my $group_by    = $rule_template->group_by;
     my $order_by    = $rule_template->order_by;
     my $aggregate   = $rule_template->aggregate;
+    my $limit       = $rule_template->limit;
 
     if (my $sub_typing_property) {
         # When the rule has a property specified which indicates a specific sub-type, catch this and re-call
@@ -492,8 +485,7 @@ sub _create_import_iterator_for_underlying_context {
             unless ($set) {
                 Carp::croak("Failed to fabricate $set_class for rule $ss_rule");
             }
-            my $aggregates = $set->{__aggregates} ||= {};
-            @$aggregates{@aggregate_properties} = @$row[$division_point+1..$#$row];
+            @$set{@aggregate_properties} = @$row[$division_point+1..$#$row];
             return $set;
         };
 
@@ -559,7 +551,7 @@ sub _create_import_iterator_for_underlying_context {
         my $primary_object_for_next_db_row;
 
         LOAD_AN_OBJECT:
-        until (defined $primary_object_for_next_db_row) { # note that we return directly when the db is out of data
+        until ($primary_object_for_next_db_row) { # note that we return directly when the db is out of data
 
             my ($next_db_row);
             ($next_db_row) = $db_iterator->() if ($db_iterator);
@@ -638,7 +630,7 @@ sub _create_import_iterator_for_underlying_context {
                         # The last time this happened, it was because a get() was done on an abstract
                         # base class with only 'id' as a param.  When the subclassified rule was
                         # turned into SQL in UR::DataSource::QueryPlan()
-                        # it removed that one 'id' filter, since it assumed any class with more than
+                        # it removed that one 'id' filter, since it assummed any class with more than
                         # one ID property (usually classes have a named whatever_id property, and an alias 'id'
                         # property) will have a rule that covered both ID properties
                         Carp::carp("Leftover objects in subordinate iterator for $class.  This shouldn't happen, but it's not fatal...");
@@ -737,15 +729,17 @@ sub _create_import_iterator_for_underlying_context {
 
             $primary_object_for_next_db_row = $imported[-1];
 
-            # The object importer will return undef for an object if no object
-            # got created for that $next_db_row, and will return a string if the object
-            # needs to be subclassed before being returned.  Don't put serial numbers on
-            # these
-            map { $_->{'__get_serial'} = $this_get_serial }
-                grep { defined && ref }
-                @imported;
+            foreach my $obj (@imported) {
+                # The object importer will return undef for an object if no object
+                # got created for that $next_db_row, and will return a string if the object
+                # needs to be subclassed before being returned.  Don't put serial numbers on
+                # these
+                next unless (defined($obj) && ref($obj));
 
-            if ($re_iterate and defined($primary_object_for_next_db_row) and ! ref($primary_object_for_next_db_row)) {
+                $obj->{'__get_serial'} = $this_get_serial;
+            }
+
+            if ($re_iterate and $primary_object_for_next_db_row and ! ref($primary_object_for_next_db_row)) {
                 # It is possible that one or more objects go into subclasses which require more
                 # data than is on the results row.  For each subclass (or set of subclasses),
                 # we make a more specific, subordinate iterator to delegate-to.
@@ -771,7 +765,7 @@ sub _create_import_iterator_for_underlying_context {
 
             } # end of handling a possible subordinate iterator delegate
 
-            unless (defined $primary_object_for_next_db_row) {
+            unless ($primary_object_for_next_db_row) {
             #if (!$primary_object_for_next_db_row or $rule->evaluate($primary_object_for_next_db_row)) {
                 redo LOAD_AN_OBJECT;
             }
@@ -786,9 +780,7 @@ sub _create_import_iterator_for_underlying_context {
                 push @by_hand_recursive_source_values, @values;
             }
 
-            if (! defined($next_object_to_return)
-                or (Scalar::Util::refaddr($next_object_to_return) == Scalar::Util::refaddr($primary_object_for_next_db_row))
-            ) {
+            if (! $next_object_to_return or $next_object_to_return eq $primary_object_for_next_db_row) {
                 # The first time through the iterator, we need to buffer the object until
                 # $primary_object_for_next_db_row is something different.
                 $next_object_to_return = $primary_object_for_next_db_row;
