@@ -4,7 +4,7 @@ package UR::Namespace::Command::Update::ClassesFromDb;
 use strict;
 use warnings;
 use UR;
-our $VERSION = "0.34"; # UR $VERSION;
+our $VERSION = "0.35"; # UR $VERSION;
 use Text::Diff;
 
 UR::Object::Type->define(
@@ -117,12 +117,17 @@ sub execute {
                 UR::Object::Type->get(data_source => $item->[0], table_name => $item->[1]);
             }
         }
-        
+
         for my $item (@$ds_table_list) {
             my ($data_source, $table_name) = @$item;
+            $self->_update_database_metadata_objects_for_schema_changes(
+                data_source => $data_source,
+                force_check_all_tables => $force_check_all_tables,
+                table_name => $table_name,
+            );
             for my $dd_class (qw/UR::DataSource::RDBMS::Table UR::DataSource::RDBMS::FkConstraint UR::DataSource::RDBMS::TableColumn/) {
                 push @data_dictionary_objects,
-                    $dd_class->get(data_source => $data_source, table_name => $table_name);
+                    $dd_class->get(data_source_obj => $data_source, table_name => $table_name);
             }
         }
     }
@@ -406,6 +411,7 @@ sub _update_database_metadata_objects_for_schema_changes {
     my ($self, %params) = @_;
     my $data_source = delete $params{data_source};
     my $force_check_all_tables = delete $params{force_check_all_tables};
+    my $table_name = delete $params{table_name};
     die "unknown params " . Dumper(\%params) if keys %params;
 
     #$data_source = $data_source->class;
@@ -428,6 +434,8 @@ sub _update_database_metadata_objects_for_schema_changes {
 
     my %all_table_names = (%current_table_names, %previous_table_names);
 
+    if($table_name) { %all_table_names = ($table_name => 1) }
+
     my $new_object_revision = $UR::Context::current->now();
 
     # handle tables which are new/updated by updating the class
@@ -874,26 +882,14 @@ sub  _update_class_metadata_objects_to_match_database_metadata_changes {
                              $table_name)
                      );
 
-            my $type_name = $data_source->resolve_type_name_for_table_name($table_name);
-            $type_name .= ' view' if ($table->table_type =~ m/view/i);
-            unless ($type_name) {
-                Carp::confess(
-                    "Failed to resolve a type name for new table "
-                    . $table_name
-                );
-            }
-
             if ($class) {
-                $class->type_name($type_name);
                 $class->doc($table->remarks ? $table->remarks: undef);
                 $class->data_source($data_source);
                 $class->table_name($table_name);
-                # FIXME we should pick one of these names to standarize on
                 $class->er_role($table->er_type);
             } else {
                 $class = UR::Object::Type->create(
                             class_name => $class_name,
-                            type_name => $type_name,
                             doc => ($table->remarks ? $table->remarks: undef),
                             data_source_id => $data_source,
                             table_name => $table_name,
@@ -956,7 +952,7 @@ sub  _update_class_metadata_objects_to_match_database_metadata_changes {
         my $class_name = $class->class_name;
         my $property;
         foreach my $prop_object ( $class->direct_property_metas ) {
-            if ($prop_object->column_name eq $column_name) {
+            if (defined $prop_object->column_name and $prop_object->column_name eq $column_name) {
                 $property = $prop_object;
                 last;
             }
@@ -1011,20 +1007,8 @@ sub  _update_class_metadata_objects_to_match_database_metadata_changes {
                 );
             }
 
-            my $attribute_name = $data_source->resolve_attribute_name_for_column_name($column->column_name);
-            unless ($attribute_name) {
-                Carp::confess(
-                    "Failed to resolve a attribute name for new column "
-                    . $column->column_name
-                );
-            }
-
-            my $type_name = $class->type_name;
-
             $property = UR::Object::Property->create(
                 class_name     => $class_name,
-                type_name      => $type_name,
-                attribute_name => $attribute_name,
                 property_name  => $property_name,
                 column_name    => $column_name,
                 data_type      => $ur_data_type,
@@ -1053,15 +1037,6 @@ sub  _update_class_metadata_objects_to_match_database_metadata_changes {
                 );
             }
         }
-        
-        # FIXME Moved the creating/setting of these properties up a bit, since the
-        # create() invovles a call into the class object containing the property, which handles
-        # updating the class' flat-format data.  Changing the 
-        #$property->data_type($column->data_type);
-        #$property->data_length($column->data_length);
-        #$property->is_optional($column->nullable eq "Y" ? 1 : 0);
-        #$property->doc($column->remarks);
-
     } # next column
 
     $self->status_message("Updating class ID properties...\n");
@@ -1075,7 +1050,6 @@ sub  _update_class_metadata_objects_to_match_database_metadata_changes {
         my $class = $self->_get_class_meta_for_table_name(data_source => $table->data_source,
                                                           table_name => $table_name);
         my $class_name = $class->class_name;
-        my $type_name = $class->type_name;
         my @properties = UR::Object::Property->get(class_name => $class_name);
 
         unless (@properties) {
@@ -1163,7 +1137,6 @@ sub  _update_class_metadata_objects_to_match_database_metadata_changes {
         my $class = $self->_get_class_meta_for_table_name(data_source => $table->data_source,
                                                           table_name => $table->table_name);
         my $class_name = $class->class_name;
-        my $type_name = $class->type_name;
 
         my @properties = UR::Object::Property->get(class_name => $class_name);
 
@@ -1231,10 +1204,6 @@ sub  _update_class_metadata_objects_to_match_database_metadata_changes {
         my $class_name = $class->class_name;
         my $r_class_name = $r_class->class_name;
 
-        my $type_name = $class->type_name;
-        my $r_type_name = $r_class->type_name;
-
-
         # Create an object-accessor property to go with this FK
         # First we have to figure out a proper delegation name
         # which is a rather convoluted process
@@ -1305,12 +1274,10 @@ sub  _update_class_metadata_objects_to_match_database_metadata_changes {
         {
             my $column_name = $column_names[$i];
             my $property = $properties[$i];
-            my $attribute_name = $property->attribute_name;
             my $property_name = $property_names[$i];
 
             my $r_column_name = $r_column_names[$i];
             my $r_property = $r_properties[$i];
-            my $r_attribute_name = $r_property->attribute_name;
             my $r_property_name = $r_property_names[$i];
         }
 
