@@ -3,7 +3,7 @@ package UR::Namespace::View::SchemaBrowser::CgiApp::Class;
 use strict;
 use warnings;
 require UR;
-our $VERSION = "0.34"; # UR $VERSION;
+our $VERSION = "0.35"; # UR $VERSION;
 
 use base 'UR::Namespace::View::SchemaBrowser::CgiApp::Base';
 
@@ -60,7 +60,7 @@ my $self = shift;
     if ($class_obj) {
         my @class_detail;
         foreach my $prop_name ( qw( namespace doc er_role is_abstract is_final is_singleton
-                                    sub_classification_meta_class_name subclassify_by type_name) ) {
+                                    sub_classification_meta_class_name subclassify_by) ) {
 
             push @class_detail, { PROPERTY_NAME => $prop_name, PROPERTY_VALUE => $class_obj->$prop_name };
         }
