@@ -8,7 +8,7 @@ require Term::ANSIColor;
 use UR;
 use UR::Object::Command::List::Style;
 
-our $VERSION = "0.34"; # UR $VERSION;
+our $VERSION = "0.35"; # UR $VERSION;
 
 class UR::Object::Command::List {
     is => 'Command',
@@ -20,12 +20,13 @@ class UR::Object::Command::List {
         filter => {
             is => 'Text',  
             is_optional => 1,
-            doc => 'Filter results based on the parameters.  See below for how to.'
+            doc => 'Filter results based on the parameters.  See below for how to.',
+            shell_args_position => 1,
         },
         show => {
             is => 'Text',
             is_optional => 1,
-            doc => 'Specify which columns to show, in order.' 
+            doc => 'Specify which columns to show, in order.',
         },
         order_by => {
             is => 'Text',
@@ -60,10 +61,10 @@ class UR::Object::Command::List {
         _fields => {
             is_many => 1,
             is_optional => 1,
-            doc => 'Methods which the caller intends to use on the fetched objects.  May lead to pre-fetching the data.'
+            doc => 'Methods which the caller intends to use on the fetched objects.  May lead to pre-fetching the data.',
         },
     ], 
-    doc => 'lists objects matching specified params'
+    doc => 'lists objects matching specified params',
 };
 
 sub sub_command_sort_position { .2 };
@@ -138,10 +139,9 @@ sub execute {
     my $bool_expr = $self->_resolve_boolexpr();
     return unless (defined $bool_expr);
   
-    # preloading the data ensures that the iterator doesn't trigger requery
-    my @results = $self->subject_class_name->get($bool_expr);
-
-    # TODO: remove the iterator entirely from the lister since all of the data is above--ss
+    # TODO: instead of using an iterator, get all the results back in a list and
+    # have the styler use the list, since it needs all the results to space the columns
+    # out properly anyway
     my $iterator;
     unless ($iterator = $self->subject_class_name->create_iterator(where => $bool_expr)) {
         $self->error_message($self->subject_class_name->error_message);
