@@ -7,7 +7,7 @@ package UR::Object::Type;
 use strict;
 use warnings;
 require UR;
-our $VERSION = "0.34"; # UR $VERSION;
+our $VERSION = "0.35"; # UR $VERSION;
 
 use Carp ();
 use Sub::Name ();
@@ -454,7 +454,7 @@ sub _normalize_class_description_impl {
         $new_class{er_role} = $UR::Object::Type::defaults{'er_role'};
     }   
  
-    my @crap = qw/source short_name/;
+    my @crap = qw/source/;
     delete @old_class{@crap};
     
     if ($class_name =~ /^(.*?)::/) {
@@ -666,6 +666,8 @@ sub _normalize_class_description_impl {
                              and
                              exists($properties->{$name}->{'is_optional'})
                              and
+                             defined($properties->{$name}->{'is_optional'})
+                             and
                              $properties->{$name}->{'is_optional'} == 0);
                     $properties->{$name}->{$key} = $params->{$key};
                 }
@@ -756,10 +758,6 @@ sub _normalize_class_description_impl {
         }
     }
 
-    unless ($new_class{type_name}) {
-        $new_class{type_name} = $new_class{class_name};
-    }
-    
     if (($new_class{data_source_id} and not ref($new_class{data_source_id})) and not $new_class{schema_name}) {
         my $s = $new_class{data_source_id};
         $s =~ s/^.*::DataSource:://;
@@ -954,15 +952,12 @@ sub _normalize_property_description1 {
     my %new_property = (
         class_name => $class_name,
         property_name => $property_name,
-        type_name => $new_class{type_name},
     );
     
     for my $mapping (
         [ property_type                   => qw/resolution/],
         [ class_name                      => qw//],
         [ property_name                   => qw//],
-        [ type_name                       => qw//],
-        [ attribute_name                  => qw//],
         [ column_name                     => qw/sql/],
         [ constraint_name                 => qw//],
         [ data_length                     => qw/len/],
@@ -1009,16 +1004,15 @@ sub _normalize_property_description1 {
     ) {
         my ($primary_field_name, @alternate_field_names) = @$mapping;
         my @all_fields = ($primary_field_name, @alternate_field_names);
-        my @values = grep { defined($_) } delete @old_property{@all_fields};
-        if (@values > 1) {
-            Carp::confess(
-                "Multiple values in class definition for $class_name property $property_name.  Field "
-                . join("/", @all_fields)
-                . " has values "
-                . Data::Dumper::Dumper(\@values)
-            );
+
+        my @keys = grep { exists $old_property{$_} } @all_fields;
+        if (@keys > 1) {
+            Carp::croak("Invalid class definition for $class_name in property '$property_name'.  The keys "
+                        . join(', ',@keys) . " are all synonyms for $primary_field_name");
+
         }
-        elsif (@values == 1) {
+        my @values = grep { defined } delete @old_property{@all_fields};
+        if (@keys == 1) {
             $new_property{$primary_field_name} = $values[0];
         }
 
@@ -1107,7 +1101,10 @@ sub _normalize_property_description2 {
         $the_data_source = $new_class{'data_source_id'}->{'is'};
     } elsif ($new_class{'data_source_id'}) {
         $the_data_source = $new_class{'data_source_id'};
-        $the_data_source = UR::DataSource->get($the_data_source) || $the_data_source->get();
+        $the_data_source = UR::DataSource->get($the_data_source) || eval { $the_data_source->get() };
+        unless ($the_data_source) {
+            Carp::croak("Can't resolve data source from value '".$new_class{'data_source_id'}."' in class definition for $class_name");
+        }
     }
     # UR::DataSource::File-backed classes don't have table_names, but for querying/saving to
     # work property, their properties still have to have column_name filled in
@@ -1124,11 +1121,6 @@ sub _normalize_property_description2 {
         }
     }
     
-    unless ($new_property{attribute_name}) {
-        $new_property{attribute_name} = $property_name;
-        $new_property{attribute_name} =~ s/_/ /g;
-    }
-
     if ($new_property{order_by} and not $new_property{is_many}) {
         die "Cannot use order_by except on is_many properties!";
     }
@@ -1254,7 +1246,6 @@ sub _complete_class_meta_object_definitions {
 
     # grab some data from the object
     my $class_name = $self->{class_name};
-    my $type_name = $self->{type_name};
     my $table_name = $self->{table_name};    
     
     # decompose the embedded complex data structures into normalized objects
@@ -1312,13 +1303,7 @@ sub _complete_class_meta_object_definitions {
             $self->error_message("Failed to find parent class $parent_class_name\n");
             return;
         }
-        
-        unless(ref($parent_class) and $parent_class->can('type_name')) {            
-            print Data::Dumper::Dumper($parent_class);
-            #$DB::single = 1;
-            redo;
-        }
-        
+
         if (not defined $self->schema_name) {
             if (my $schema_name = $parent_class->schema_name) {
                 $self->{'schema_name'} = $self->{'db_committed'}->{'schema_name'} = $schema_name;
@@ -1404,7 +1389,6 @@ sub _complete_class_meta_object_definitions {
     }
     
     # make old-style (bc4nf) property objects in the default way
-    $type_name = $self->{type_name};
     my @property_objects;
     
     for my $pinfo (values %$properties) {                        
@@ -1477,7 +1461,7 @@ sub _complete_class_meta_object_definitions {
             }
             else {
                 $properties = @$unique_set;
-                $name = $type_name . "_$n";
+                $name = '(unnamed)';
                 $n++;
             }
             for my $property_name (sort @$properties) {
