@@ -3,11 +3,15 @@ package UR::Object::View::Default::Gtk;
 use strict;
 use warnings;
 require UR;
-our $VERSION = "0.34"; # UR $VERSION;
+our $VERSION = "0.35"; # UR $VERSION;
 
 UR::Object::Type->define(
     class_name => __PACKAGE__,
     is => 'UR::Object::View',
+    has_constant => [
+        perspective => { value => 'default' },
+        toolkit => { value => 'gtk' },
+    ]
 );
 
 sub _create_widget {
