@@ -3,13 +3,13 @@ package UR::Object::View::Default::Gtk2;
 use strict;
 use warnings;
 require UR;
-our $VERSION = "0.34"; # UR $VERSION;
+our $VERSION = "0.35"; # UR $VERSION;
 
 class UR::Object::View::Default::Gtk2 {
     is => 'UR::Object::View',
     has_constant => [
-        perspective => 'default',
-        toolkit => 'gtk2'
+        perspective => { value => 'default'},
+        toolkit => { value => 'gtk2'},
     ],
 };
 
