@@ -4,11 +4,11 @@ use strict;
 use warnings;
 
 require UR;
-our $VERSION = "0.34"; # UR $VERSION;
+our $VERSION = "0.35"; # UR $VERSION;
 
 UR::Object::Type->define(
     class_name => 'UR::Value::SCALAR',
-    is => ['UR::Value'],
+    is => ['UR::Value::PerlReference'],
 );
 
 1;
